@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,16 +19,17 @@
 
 """
 
+import collections
 import hashlib
 import sys
-from abc import ABCMeta, abstractmethod
+from abc import abstractmethod
 
 from nuitka import Options
-from nuitka.__past__ import iterItems
+from nuitka.__past__ import getMetaClassBase, iterItems
 from nuitka.Builtins import (
     builtin_anon_codes,
     builtin_anon_values,
-    builtin_exception_values_list
+    builtin_exception_values_list,
 )
 from nuitka.PythonVersions import python_version
 from nuitka.utils.InstanceCounters import counted_del, counted_init
@@ -36,22 +37,9 @@ from nuitka.utils.InstanceCounters import counted_del, counted_init
 from .Namify import namifyConstant
 from .VariableDeclarations import VariableDeclaration, VariableStorage
 
-
-class ContextMetaClass(ABCMeta):
-    pass
-
-
-# For Python2/3 compatible source, we create a base class that has the metaclass
-# used and doesn't require making a choice.
-ContextMetaClassBase = ContextMetaClass(
-    "ContextMetaClassBase",
-    (object,),
-    {}
-)
-
-
 # Many methods won't use self, but it's the interface. pylint: disable=no-self-use
 
+
 class TempMixin(object):
     # Lots of details, everything gets to store bits here, to indicate
     # code generation states, and there are many, pylint: disable=too-many-instance-attributes
@@ -79,20 +67,13 @@ class TempMixin(object):
 
     def _formatTempName(self, base_name, number):
         if number is None:
-            return "tmp_{name}".format(
-                name = base_name
-            )
+            return "tmp_{name}".format(name=base_name)
         else:
-            return "tmp_{name}_{number:d}".format(
-                name   = base_name,
-                number = number
-            )
+            return "tmp_{name}_{number:d}".format(name=base_name, number=number)
 
-    def allocateTempName(self, base_name, type_name = "PyObject *",
-                         unique = False):
+    def allocateTempName(self, base_name, type_name="PyObject *", unique=False):
         # We might be hard coding too many details for special temps
-        # here.
-
+        # here, pylint: disable=too-many-branches
 
         if unique:
             number = None
@@ -102,10 +83,7 @@ class TempMixin(object):
 
         self.tmp_names[base_name] = number
 
-        formatted_name = self._formatTempName(
-            base_name = base_name,
-            number    = number
-        )
+        formatted_name = self._formatTempName(base_name=base_name, number=number)
 
         if unique:
             result = self.variable_storage.getVariableDeclarationTop(formatted_name)
@@ -122,23 +100,25 @@ class TempMixin(object):
 
                 if base_name == "unused":
                     result = self.variable_storage.addVariableDeclarationFunction(
-                        c_type     = type_name,
-                        code_name  = formatted_name,
-                        init_value = init_value
+                        c_type=type_name,
+                        code_name=formatted_name,
+                        init_value=init_value,
                     )
                 else:
                     result = self.variable_storage.addVariableDeclarationTop(
-                        c_type     = type_name,
-                        code_name  = formatted_name,
-                        init_value = init_value
+                        c_type=type_name,
+                        code_name=formatted_name,
+                        init_value=init_value,
                     )
             else:
+                if type_name.startswith("NUITKA_MAY_BE_UNUSED"):
+                    type_name = type_name[21:]
+
                 assert result.c_type == type_name
 
         else:
             result = self.variable_storage.addVariableDeclarationLocal(
-                c_type    = type_name,
-                code_name = formatted_name
+                c_type=type_name, code_name=formatted_name
             )
 
         return result
@@ -149,10 +129,10 @@ class TempMixin(object):
         self.tmp_names[base_name] = number
 
     def getIntResName(self):
-        return self.allocateTempName("res", "int", unique = True)
+        return self.allocateTempName("res", "int", unique=True)
 
     def getBoolResName(self):
-        return self.allocateTempName("result", "bool", unique = True)
+        return self.allocateTempName("result", "bool", unique=True)
 
     def hasTempName(self, base_name):
         return base_name in self.tmp_names
@@ -186,10 +166,7 @@ class TempMixin(object):
         result += 1
         self.labels[label] = result
 
-        return "{name}_{number:d}".format(
-            name   = label,
-            number = result
-        )
+        return "{name}_{number:d}".format(name=label, number=result)
 
     def getLabelCount(self, label):
         return self.labels.get(label, 0)
@@ -210,23 +187,23 @@ class TempMixin(object):
             self.variable_storage.addVariableDeclarationTop(
                 "PyObject *",
                 "exception_keeper_type_%d" % self.keeper_variable_count,
-                keeper_obj_init
+                keeper_obj_init,
             ),
             self.variable_storage.addVariableDeclarationTop(
                 "PyObject *",
                 "exception_keeper_value_%d" % self.keeper_variable_count,
-                keeper_obj_init
+                keeper_obj_init,
             ),
             self.variable_storage.addVariableDeclarationTop(
                 "PyTracebackObject *",
                 "exception_keeper_tb_%d" % self.keeper_variable_count,
-                keeper_obj_init
+                keeper_obj_init,
             ),
             self.variable_storage.addVariableDeclarationTop(
                 "NUITKA_MAY_BE_UNUSED int",
                 "exception_keeper_lineno_%d" % self.keeper_variable_count,
-                '0' if debug else None
-            )
+                "0" if debug else None,
+            ),
         )
 
     def getExceptionKeeperVariables(self):
@@ -253,18 +230,18 @@ class TempMixin(object):
                 self.variable_storage.addVariableDeclarationTop(
                     "PyObject *",
                     "exception_preserved_type_%d" % preserver_id,
-                    preserver_obj_init
+                    preserver_obj_init,
                 ),
                 self.variable_storage.addVariableDeclarationTop(
                     "PyObject *",
                     "exception_preserved_value_%d" % preserver_id,
-                    preserver_obj_init
+                    preserver_obj_init,
                 ),
                 self.variable_storage.addVariableDeclarationTop(
                     "PyTracebackObject *",
                     "exception_preserved_tb_%d" % preserver_id,
-                    preserver_obj_init
-                )
+                    preserver_obj_init,
+                ),
             )
 
         return self.preserver_variable_declaration[preserver_id]
@@ -286,6 +263,7 @@ class TempMixin(object):
 
     def addCleanupTempName(self, tmp_name):
         assert tmp_name not in self.cleanup_names[-1], tmp_name
+        assert tmp_name.c_type != "void"
 
         self.cleanup_names[-1].append(tmp_name)
 
@@ -304,6 +282,27 @@ class TempMixin(object):
         del self.cleanup_names[-1]
 
 
+CodeObjectHandle = collections.namedtuple(
+    "CodeObjectHandle",
+    (
+        "co_name",
+        "co_kind",
+        "co_varnames",
+        "co_argcount",
+        "co_posonlyargcount",
+        "co_kwonlyargcount",
+        "co_has_starlist",
+        "co_has_stardict",
+        "co_filename",
+        "line_number",
+        "future_flags",
+        "co_new_locals",
+        "has_closure",
+        "is_optimized",
+    ),
+)
+
+
 class CodeObjectsMixin(object):
     def __init__(self):
         # Code objects needed made unique by a key.
@@ -313,20 +312,21 @@ class CodeObjectsMixin(object):
         return sorted(iterItems(self.code_objects))
 
     def getCodeObjectHandle(self, code_object):
-        key = (
-            code_object.getFilename(),
-            code_object.getCodeObjectName(),
-            code_object.getLineNumber(),
-            code_object.getVarNames(),
-            code_object.getArgumentCount(),
-            code_object.getKwOnlyParameterCount(),
-            code_object.getCodeObjectKind(),
-            code_object.getFlagIsOptimizedValue(),
-            code_object.getFlagNewLocalsValue(),
-            code_object.hasStarListArg(),
-            code_object.hasStarDictArg(),
-            code_object.getFlagHasClosureValue(),
-            code_object.getFutureSpec().asFlags()
+        key = CodeObjectHandle(
+            co_filename=code_object.getFilename(),
+            co_name=code_object.getCodeObjectName(),
+            line_number=code_object.getLineNumber(),
+            co_varnames=code_object.getVarNames(),
+            co_argcount=code_object.getArgumentCount(),
+            co_posonlyargcount=code_object.getPosOnlyParameterCount(),
+            co_kwonlyargcount=code_object.getKwOnlyParameterCount(),
+            co_kind=code_object.getCodeObjectKind(),
+            is_optimized=code_object.getFlagIsOptimizedValue(),
+            co_new_locals=code_object.getFlagNewLocalsValue(),
+            co_has_starlist=code_object.hasStarListArg(),
+            co_has_stardict=code_object.hasStarDictArg(),
+            has_closure=code_object.getFlagHasClosureValue(),
+            future_flags=code_object.getFutureSpec().asFlags(),
         )
 
         if key not in self.code_objects:
@@ -335,22 +335,21 @@ class CodeObjectsMixin(object):
         return self.code_objects[key]
 
     if python_version < 300:
+
         def _calcHash(self, key):
-            hash_value = hashlib.md5(
-                "%s-%s-%d-%s-%d-%d-%s-%s-%s-%s-%s-%s-%s" % key
-            )
+            hash_value = hashlib.md5("-".join(str(s) for s in key))
 
             return hash_value.hexdigest()
+
     else:
+
         def _calcHash(self, key):
-            hash_value = hashlib.md5(
-                ("%s-%s-%d-%s-%d-%d-%s-%s-%s-%s-%s-%s-%s" % key).encode("utf-8")
-            )
+            hash_value = hashlib.md5("-".join(str(s) for s in key).encode("utf-8"))
 
             return hash_value.hexdigest()
 
 
-class PythonContextBase(ContextMetaClassBase):
+class PythonContextBase(getMetaClassBase("Context")):
     @counted_init
     def __init__(self):
         self.source_ref = None
@@ -377,6 +376,9 @@ class PythonContextBase(ContextMetaClassBase):
         # self.last_source_ref = None
         return result
 
+    def getInplaceLeftName(self):
+        return self.allocateTempName("inplace_orig", "PyObject *", True)
+
     @abstractmethod
     def getConstantCode(self, constant):
         pass
@@ -426,8 +428,7 @@ class PythonContextBase(ContextMetaClassBase):
         pass
 
     @abstractmethod
-    def allocateTempName(self, base_name, type_name = "PyObject *",
-                         unique = False):
+    def allocateTempName(self, base_name, type_name="PyObject *", unique=False):
         pass
 
     @abstractmethod
@@ -532,7 +533,7 @@ class PythonContextBase(ContextMetaClassBase):
 
 
 class PythonChildContextBase(PythonContextBase):
-    # Base classes can be abstract, pylint: disable=abstract-method
+    # Base classes can be abstract, pylint: disable=I0021,abstract-method
 
     def __init__(self, parent):
         PythonContextBase.__init__(self)
@@ -591,11 +592,12 @@ def _getConstantDefaultPopulation():
         False,
         0,
         1,
-
+        # Some math operations shortcut to these
+        0.0,
+        -0.0,
         # For Python3 empty bytes, no effect for Python2, same as "", used for
         # code objects.
         b"",
-
         # Python mechanics, used in various helpers.
         "__module__",
         "__class__",
@@ -612,10 +614,10 @@ def _getConstantDefaultPopulation():
         "__all__",
         "__cmp__",
         "__iter__",
-
+        # Nuitka specific
+        "__compiled__",
         # Patched module name.
         "inspect",
-
         # Names of built-ins used in helper code.
         "compile",
         "range",
@@ -626,49 +628,34 @@ def _getConstantDefaultPopulation():
         "bytearray",
         "staticmethod",
         "classmethod",
-
         # Arguments of __import__ built-in used in helper code.
         "name",
         "globals",
         "locals",
         "fromlist",
         "level",
-
         # Meta path based loader.
         "read",
-        "rb"
+        "rb",
     ]
 
     if python_version >= 300:
         # For Python3 modules
-        result += (
-            "__cached__",
-            "__loader__",
-        )
+        result += ("__cached__", "__loader__")
 
         # For Python3 print
-        result += (
-            "print",
-            "end",
-            "file"
-        )
+        result += ("print", "end", "file")
 
         # For Python3 "bytes" built-in.
-        result.append(
-            "bytes"
-        )
-
-        # For Python3 "__name__" to "__package__" parsing
-        result.append(
-            '.'
-        )
+        result.append("bytes")
 
+    # For meta path based loader, iter_modules and Python3 "__name__" to
+    # "__package__" parsing
+    result.append(".")
 
     if python_version >= 300:
         # Modules have that attribute starting with 3.3
-        result.append(
-            "__loader__"
-        )
+        result.append("__loader__")
 
     if python_version >= 340:
         result.append(
@@ -683,83 +670,44 @@ def _getConstantDefaultPopulation():
             "close",
         )
 
-
     if python_version < 300:
         # For patching Python2 internal class type
-        result += (
-            "__getattr__",
-            "__setattr__",
-            "__delattr__",
-        )
+        result += ("__getattr__", "__setattr__", "__delattr__")
 
         # For patching Python2 "sys" attributes for current exception
-        result += (
-            "exc_type",
-            "exc_value",
-            "exc_traceback"
-        )
+        result += ("exc_type", "exc_value", "exc_traceback")
 
     # The xrange built-in is Python2 only.
     if python_version < 300:
-        result.append(
-            "xrange"
-        )
+        result.append("xrange")
 
     # Executables only
     if not Options.shallMakeModule():
-        result.append(
-            "__main__"
-        )
+        result.append("__main__")
 
         # The "site" module is referenced in inspect patching.
-        result.append(
-            "site"
-        )
+        result.append("site")
 
     # Built-in original values
     if not Options.shallMakeModule():
-        result += [
-            "type",
-            "len",
-            "range",
-            "repr",
-            "int",
-            "iter",
-        ]
+        result += ["type", "len", "range", "repr", "int", "iter"]
 
         if python_version < 300:
-            result.append(
-                "long",
-            )
-
-    # Disabling warnings at startup
-    if "no_warnings" in Options.getPythonFlags():
-        result.append(
-            "ignore"
-        )
+            result.append("long")
 
     if python_version >= 340:
         # Setting the __spec__ module attribute.
-        result += [
-            "__spec__",
-            "_initializing"
-        ]
+        result += ["__spec__", "_initializing", "submodule_search_locations"]
 
     if python_version >= 350:
         # Patching the types module.
-        result.append(
-            "types"
-        )
+        result.append("types")
 
     if not Options.shallMakeModule():
-        result.append(
-            sys.executable
-        )
+        result += [sys.executable, sys.prefix]
 
     if python_version >= 370:
-        result.append(
-            "__class_getitem__"
-        )
+        result.append("__class_getitem__")
 
     return result
 
@@ -809,9 +757,15 @@ class PythonGlobalContext(object):
                 key = "(PyObject *)&PyFrozenSet_Type"
             elif python_version >= 270 and constant is memoryview:
                 key = "(PyObject *)&PyMemoryView_Type"
-            elif python_version < 300 and constant is basestring: # pylint: disable=I0021,undefined-variable
+            elif (
+                python_version < 300
+                and constant is basestring  # pylint: disable=I0021,undefined-variable
+            ):
                 key = "(PyObject *)&PyBaseString_Type"
-            elif python_version < 300 and constant is xrange: # pylint: disable=I0021,undefined-variable
+            elif (
+                python_version < 300
+                and constant is xrange  # pylint: disable=I0021,undefined-variable
+            ):
                 key = "(PyObject *)&PyRange_Type"
             elif constant in builtin_anon_values:
                 key = "(PyObject *)" + builtin_anon_codes[builtin_anon_values[constant]]
@@ -864,7 +818,6 @@ class FrameDeclarationsMixin(object):
 
         self.locals_dict_names = set()
 
-
     def getFrameHandle(self):
         return self.frame_stack[-1]
 
@@ -876,7 +829,7 @@ class FrameDeclarationsMixin(object):
                 "struct Nuitka_FrameObject *",
                 "m_frame",
                 None,
-                self.getContextObjectName()
+                self.getContextObjectName(),
             )
 
         else:
@@ -886,18 +839,15 @@ class FrameDeclarationsMixin(object):
                 frame_handle += "_%d" % self.frames_used
 
             frame_identifier = self.variable_storage.addVariableDeclarationTop(
-                "struct Nuitka_FrameObject *",
-                frame_handle,
-                None
+                "struct Nuitka_FrameObject *", frame_handle, None
             )
 
         self.variable_storage.addVariableDeclarationTop(
             "NUITKA_MAY_BE_UNUSED char const *",
             "type_description_%d" % self.frames_used,
-            "NULL"
+            "NULL",
         )
 
-
         self.frame_stack.append(frame_identifier)
         return frame_identifier
 
@@ -926,7 +876,7 @@ class FrameDeclarationsMixin(object):
         # TODO: Change value of that dict to take advantage of declaration.
         self.frame_variable_types[variable] = (
             str(variable_declaration),
-            variable_declaration.getCType().getTypeIndicator()
+            variable_declaration.getCType().getTypeIndicator(),
         )
 
     def getFrameVariableTypeDescriptions(self):
@@ -939,9 +889,8 @@ class FrameDeclarationsMixin(object):
 
     def getFrameVariableTypeDescription(self):
         result = "".join(
-            self.frame_variable_types.get(variable, ("NULL", 'N'))[1]
-            for variable in
-            self.frame_variables_stack[-1]
+            self.frame_variable_types.get(variable, ("NULL", "N"))[1]
+            for variable in self.frame_variables_stack[-1]
         )
 
         if result:
@@ -953,9 +902,11 @@ class FrameDeclarationsMixin(object):
         result = []
 
         for variable in self.frame_variables_stack[-1]:
-            variable_code_name, variable_code_type = self.frame_variable_types.get(variable, ("NULL", 'N'))
+            variable_code_name, variable_code_type = self.frame_variable_types.get(
+                variable, ("NULL", "N")
+            )
 
-            if variable_code_type in ('b',):
+            if variable_code_type in ("b",):
                 result.append("(int)" + variable_code_name)
             else:
                 result.append(variable_code_name)
@@ -970,15 +921,14 @@ class FrameDeclarationsMixin(object):
 
         if result is None:
             result = self.variable_storage.addVariableDeclarationTop(
-                "PyObject *",
-                locals_dict_name,
-                "NULL"
+                "PyObject *", locals_dict_name, "NULL"
             )
 
         self.locals_dict_names.add(result)
 
         return result
 
+
 class ReturnReleaseModeMixin(object):
     def __init__(self):
         self.return_release_mode = False
@@ -1008,7 +958,7 @@ class ReturnValueNameMixin(object):
 
     def getReturnValueName(self):
         if self.return_name is None:
-            self.return_name = self.allocateTempName("return_value", unique = True)
+            self.return_name = self.allocateTempName("return_value", unique=True)
 
         return self.return_name
 
@@ -1018,9 +968,14 @@ class ReturnValueNameMixin(object):
         return result
 
 
-class PythonModuleContext(TempMixin, CodeObjectsMixin,
-                          FrameDeclarationsMixin, ReturnReleaseModeMixin,
-                          ReturnValueNameMixin, PythonContextBase):
+class PythonModuleContext(
+    TempMixin,
+    CodeObjectsMixin,
+    FrameDeclarationsMixin,
+    ReturnReleaseModeMixin,
+    ReturnValueNameMixin,
+    PythonContextBase,
+):
     # Plenty of attributes, because it's storing so many different things.
     # pylint: disable=too-many-instance-attributes
 
@@ -1051,9 +1006,7 @@ class PythonModuleContext(TempMixin, CodeObjectsMixin,
 
         self.needs_module_filename_object = False
 
-        self.variable_storage = VariableStorage(
-            heap_name = None
-        )
+        self.variable_storage = VariableStorage(heap_name=None)
 
     def __repr__(self):
         return "<PythonModuleContext instance for module %s>" % self.filename
@@ -1089,7 +1042,7 @@ class PythonModuleContext(TempMixin, CodeObjectsMixin,
     def addHelperCode(self, key, code):
         assert key not in self.helper_codes, key
 
-        self.helper_codes[ key ] = code
+        self.helper_codes[key] = code
 
     def hasHelperCode(self, key):
         return key in self.helper_codes
@@ -1100,7 +1053,7 @@ class PythonModuleContext(TempMixin, CodeObjectsMixin,
     def addDeclaration(self, key, code):
         assert key not in self.declaration_codes
 
-        self.declaration_codes[ key ] = code
+        self.declaration_codes[key] = code
 
     def getDeclarations(self):
         return self.declaration_codes
@@ -1127,14 +1080,15 @@ class PythonModuleContext(TempMixin, CodeObjectsMixin,
         return self.needs_module_filename_object
 
 
-class PythonFunctionContext(FrameDeclarationsMixin,
-                            TempMixin, ReturnReleaseModeMixin,
-                            ReturnValueNameMixin, PythonChildContextBase):
+class PythonFunctionContext(
+    FrameDeclarationsMixin,
+    TempMixin,
+    ReturnReleaseModeMixin,
+    ReturnValueNameMixin,
+    PythonChildContextBase,
+):
     def __init__(self, parent, function):
-        PythonChildContextBase.__init__(
-            self,
-            parent = parent
-        )
+        PythonChildContextBase.__init__(self, parent=parent)
 
         TempMixin.__init__(self)
         FrameDeclarationsMixin.__init__(self)
@@ -1151,15 +1105,13 @@ class PythonFunctionContext(FrameDeclarationsMixin,
         self.variable_storage = self._makeVariableStorage()
 
     def _makeVariableStorage(self):
-        return VariableStorage(
-            heap_name = None
-        )
+        return VariableStorage(heap_name=None)
 
     def __repr__(self):
         return "<%s for %s '%s'>" % (
             self.__class__.__name__,
             "function" if not self.function.isExpressionClassBody() else "class",
-            self.function.getName()
+            self.function.getName(),
         )
 
     def getFunction(self):
@@ -1192,9 +1144,7 @@ class PythonFunctionDirectContext(PythonFunctionContext):
 
 class PythonGeneratorObjectContext(PythonFunctionContext):
     def _makeVariableStorage(self):
-        return VariableStorage(
-            heap_name = "%s_heap" % self.getContextObjectName()
-        )
+        return VariableStorage(heap_name="%s_heap" % self.getContextObjectName())
 
     def isForDirectCall(self):
         return False
@@ -1210,17 +1160,9 @@ class PythonGeneratorObjectContext(PythonFunctionContext):
 
     def getGeneratorReturnValueName(self):
         if python_version >= 300:
-            return self.allocateTempName(
-                "return_value",
-                "PyObject *",
-                unique = True
-            )
+            return self.allocateTempName("return_value", "PyObject *", unique=True)
         else:
-            return self.allocateTempName(
-                "generator_return",
-                "bool",
-                unique = True
-            )
+            return self.allocateTempName("generator_return", "bool", unique=True)
 
 
 class PythonCoroutineObjectContext(PythonGeneratorObjectContext):
@@ -1241,13 +1183,11 @@ class PythonFunctionCreatedContext(PythonFunctionContext):
         return True
 
 
-class PythonFunctionOutlineContext(ReturnReleaseModeMixin,
-                                   ReturnValueNameMixin, PythonChildContextBase):
+class PythonFunctionOutlineContext(
+    ReturnReleaseModeMixin, ReturnValueNameMixin, PythonChildContextBase
+):
     def __init__(self, parent, outline):
-        PythonChildContextBase.__init__(
-            self,
-            parent = parent
-        )
+        PythonChildContextBase.__init__(self, parent=parent)
 
         ReturnReleaseModeMixin.__init__(self)
         ReturnValueNameMixin.__init__(self)
@@ -1265,8 +1205,7 @@ class PythonFunctionOutlineContext(ReturnReleaseModeMixin,
     def allocateLabel(self, label):
         return self.parent.allocateLabel(label)
 
-    def allocateTempName(self, base_name, type_name = "PyObject *",
-                         unique = False):
+    def allocateTempName(self, base_name, type_name="PyObject *", unique=False):
         return self.parent.allocateTempName(base_name, type_name, unique)
 
     def skipTempName(self, base_name):
