@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,25 +22,28 @@
 
 template_iterator_check = """\
 // Check if iterator has left-over elements.
-assertObject( %(iterator_name)s ); assert( HAS_ITERNEXT( %(iterator_name)s ) );
+CHECK_OBJECT( %(iterator_name)s ); assert( HAS_ITERNEXT( %(iterator_name)s ) );
 
 %(attempt_name)s = (*Py_TYPE( %(iterator_name)s )->tp_iternext)( %(iterator_name)s );
 
 if (likely( %(attempt_name)s == NULL ))
 {
-    // TODO: Could first fetch, then check, should be faster.
-    if ( !ERROR_OCCURRED() )
+    PyObject *error = GET_ERROR_OCCURRED();
+
+    if ( error != NULL )
     {
-    }
-    else if ( PyErr_ExceptionMatches( PyExc_StopIteration ))
-    {
-        PyErr_Clear();
-    }
-    else
-    {
-        PyErr_Fetch( &exception_type, &exception_value, (PyObject **)&exception_tb );
+        if ( EXCEPTION_MATCH_BOOL_SINGLE( error, PyExc_StopIteration ))
+        {
+            CLEAR_ERROR_OCCURRED();
+        }
+        else
+        {
+            FETCH_ERROR_OCCURRED( &%(exception_type)s, &%(exception_value)s, &%(exception_tb)s );
 %(release_temps_1)s
-        goto %(exception_exit)s;
+%(var_description_code_1)s
+%(line_number_code_1)s
+            goto %(exception_exit)s;
+        }
     }
 }
 else
@@ -53,7 +56,32 @@ else
 #else
     PyErr_Format( PyExc_ValueError, "too many values to unpack (expected %(count)d)" );
 #endif
-    PyErr_Fetch( &exception_type, &exception_value, (PyObject **)&exception_tb );
+    FETCH_ERROR_OCCURRED( &%(exception_type)s, &%(exception_value)s, &%(exception_tb)s );
 %(release_temps_2)s
+%(var_description_code_2)s
+%(line_number_code_2)s
     goto %(exception_exit)s;
 }"""
+
+template_loop_break_next = """\
+if ( %(to_name)s == NULL )
+{
+    if ( CHECK_AND_CLEAR_STOP_ITERATION_OCCURRED() )
+    {
+%(break_indicator_code)s
+        goto %(break_target)s;
+    }
+    else
+    {
+%(release_temps)s
+        FETCH_ERROR_OCCURRED( &%(exception_type)s, &%(exception_value)s, &%(exception_tb)s );
+%(var_description_code)s
+%(line_number_code)s
+        goto %(exception_target)s;
+    }
+}
+"""
+
+from . import TemplateDebugWrapper  # isort:skip
+
+TemplateDebugWrapper.checkDebug(globals())
