@@ -524,3 +524,45 @@ elif isinstance(v, complex):
     print("Oops, must not happen.")
 else:
     print("Oops, must not happen.")
+
+# Test for any builtins
+print(any([None, None, None]))
+print(any([None, 4, None]))
+print(any([]))
+print(any([0] * 20000))
+print(any([0] * 255))
+print(any(range(260)))
+print(any(range(1, 270)))
+print(any(range(2, 1024, 5)))
+
+def S():
+    print("Yielding 40")
+    yield 40
+    print("Yielding 60")
+    yield 60
+    print("Yielding 30")
+    yield 30
+
+print(any(x > 42 for x in S()))
+
+print("Disallowed any() without args:")
+try:
+    print(any())
+except Exception as e:
+    print("caught ", repr(e))
+
+print("any() with float not iterable:")
+try:
+    print(any(1.0))
+except Exception as e:
+    print("caught ", repr(e))
+
+print("any() with float type not iterable:")
+try:
+    print(any(float))
+except Exception as e:
+    print("caught ", repr(e))
+
+print("any() with sets:")
+print(any(set([0, 1, 2, 3, 3])))
+print(any({1:"One", 2:"Two"}))
\ No newline at end of file
