@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -16,8 +16,8 @@
 #     limitations under the License.
 #
 
-print type(__builtins__)
+print( type(__builtins__) )
 
 import ErrorExitingModule
 
-print "huch!"
+print( "Should not get here!" )
